#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

#include "pca8575.h"
#include "testboard.h"

#define DEVICE "/dev/i2c-0"
#define OFFSET 4 /* Offset of our chip's address (value of jumpers, 0-7) */
#define LEDS 4 /* The number of leds we've got */

int main (int argc, char *argv[]) {
  int dev, i;
  __u32 new_data, read_data = 0;
  __u16 leds[LEDS] = {
    TESTBOARD_LED_1,
    TESTBOARD_LED_2,
    TESTBOARD_LED_3,
    TESTBOARD_LED_4
  };

  /* Open the i2c master device */
  if ((dev = open(DEVICE, O_RDWR)) < 0) {
    perror("Can't open device");
    return 1;
  }

  /* Select the right slave for our master to talk to */
  if (i2c_select_slave(dev, PCA8575_ADDRESS + OFFSET)) {
    perror("Can't select device");
    close(dev);
    return 1;
  }

#ifdef FANCY

  /* Reset the whole thing, if not powered down yet.
   * One led has to be active in order for the little
   * state machine to work.
   */
  if (pca8575_write(dev, TESTBOARD_LED_1)) {
    perror("Can't write to I/O expander");
    close(dev);
    return 1;
  }

  /* Run a fancy-patsy light show to impress the crowd
   * first.
   */
  for (i = 0; i < LEDS; i++) {
    if (pca8575_write(dev, leds[i])) {
      perror("Can't write to I/O expander");
      close(dev);
      return 1;
    }

    usleep(100000);
  }

  for (i = LEDS - 1; i >= 0; i--) {
    if (pca8575_write(dev, leds[i])) {
      perror("Can't write to I/O expander");
      close(dev);
      return 1;
    }

    usleep(100000);
  }

#endif

  /* Reset the whole thing, if not powered down yet.
   * One led has to be active in order for the little
   * state machine to work.
   */
  if (pca8575_write(dev, TESTBOARD_LED_1)) {
    perror("Can't write to I/O expander");
    close(dev);
    return 1;
  }

  /* Loop reading data, and exit when the red
   * button is pressed
   */
  while ((read_data & TESTBOARD_BUTTON_RED) != TESTBOARD_BUTTON_RED) {
    if ((read_data = pca8575_read(dev)) < 0) {
      perror("Can't read data from I/O expander");
      close(dev);
      return 1;
    }

    /* If the grey button is pressed, increment the active led,
     * if black is pressed, do the opposite.
     */
    if ((read_data & TESTBOARD_BUTTON_GREY) == TESTBOARD_BUTTON_GREY) {
      for (i = 0; i < LEDS; i++) {
        if ((read_data & leds[i]) == leds[i]) {
          new_data = (read_data ^ leds[i]) | leds[((i < LEDS - 1) ? i + 1 : 0)];
          new_data ^= TESTBOARD_BUTTON_GREY;

          if (pca8575_write(dev, new_data)) {
            perror("Can't write to I/O expander");
            close(dev);
            return 1;
          }

          continue;
        }
      }
    } else if ((read_data & TESTBOARD_BUTTON_BLACK) == TESTBOARD_BUTTON_BLACK) {
      for (i = LEDS - 1; i >= 0; i--) {
        if ((read_data & leds[i]) == leds[i]) {
          new_data = (read_data ^ leds[i]) | leds[((i > 0) ? i - 1 : LEDS - 1)];
          new_data ^= TESTBOARD_BUTTON_BLACK;

          if (pca8575_write(dev, new_data)) {
            perror("Can't write to I/O expander");
            close(dev);
            return 1;
          }

          continue;
        }
      }
    }

    usleep(50000);
  }

  /* We're done. Zero it out. */
  if (pca8575_write(dev, 0x0000)) {
    perror("Can't write to I/O expander");
    close(dev);
    return 1;
  }

  close(dev);

  return 0;
}

